      SUBROUTINE BCCLSR( RHDAT, NH , NMXTHA, NMXPHI, CLSPAR,
     &                   NMXCLS, NUMCLS, NCLSPT, INDX )
*.
*...BCCLS2   BC clustering subroutine. (INTEGER)
*.
*.
*. INPUT     : NH          : # of Hit counters
*.             RHDAT(3,NH) : Counter data (2 Dimensinal Allay)
*.                            (1,j) : Counter ID = 1000*ITHE + IPHI
*.                            (2,j) : Energy deposite 
*.                            (3,j) : Reserved for Future
*.             NMXTHA      : # of Theta binning.
*.             NMXPHI      : # of Phi binning.
*.             CLSPAR(5)   : Clustering parameter, Eth, C1, C2, C3
*.             NMXCLS      : Size of output cluster buffer.
*.
*.       Data RHDAT should be sorted according to the pulse height
*.
*. OUTPUT    : NUMCLS      : # of Cluster
*                                    = 0 No cluster.
*                                    < 0 error.
*              RHDAT(3,NH) : Re-arranged cluster data.
*              NCLSPT(2,NUMCLS) 
*.                         : Start and last address of cluster.
*.             INDX(NH)    : Sorting indx
*.
***********************************************************************
*.         Here we apply the algorythm used by JADE group.
*.          i.e.
*.           Set parent ctr. = BL1
*.            BL1 must satisfy following condition,
*.                      E(BL1) > 45 MeV.
*.           Daughter ctr. = BL2,
*.           BL2 is located next to BL1 with side face.
*.           Next find out BL3.
*.             BL3 will be selected with following conditions,
*.               a)  E(BL2) > E(BL1)*C1
*.              or
*.               b)  E(BL3) < E(BL1)*C2 .and. E(BL3) < C3*E(BL2) .
*.
*.      Parameter values
*.        C1 = 0.65
*.        C2 = 0.08
*.        C3 = 1.20
*.           are selected.
***********************************************************************
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Nakamura Isamu
*. VERSION   : 1.00
*. CREATED   : 25-Aug-94
*. LAST MOD  : 
*.
***********************************************************************
      
      IMPLICIT NONE

      INTEGER   NH,NMXTHA, NMXPHI,NMXCLS,NUMCLS
      REAL      RHDAT(3,NH)
      INTEGER   NCLSPT(2,NMXCLS)
      REAL      CLSPAR(5),LWORK1,LWORK2,LWORK3
      INTEGER   ILST(4),NB(4),ISRT(4),JLST(4),INDX(NMXCLS)
      INTEGER   I,KEND,IBL1,IBL2,IBL3,INEXT,IDPRNT,IPHI,ITH,NBTOT
      INTEGER   IB,INB,IWORK,J,JJ
      REAL      ECL1,RCUT1,RCUT2,RCUT3
      

*-- Initialize Variables
      NUMCLS = 0
      IF( NH .LE. 0 ) RETURN
      DO 1 I=1,NMXCLS
         INDX(I) = I
 1    CONTINUE
      KEND   = NH
      IBL1   = 1
      IBL2   = 1
      INDX(1) = 1
      
 100  CONTINUE
      NUMCLS = NUMCLS + 1
      IF( NUMCLS .GT. NMXCLS ) GO TO 900
      NCLSPT(1, NUMCLS) = IBL1
      
*-- Start clustering
      ECL1 = RHDAT(2,IBL1)
      IF( ECL1 .LT. CLSPAR(1) ) GO TO 850
      
*-- Set Cut Prameter
      RCUT1 = ECL1*CLSPAR(2)
      RCUT2 = ECL1*CLSPAR(3)
      
*-- INEXT is the address of the 1-st non-clusterized member
      INEXT  = IBL1 + 1
      IF(INEXT .GT. KEND ) GO TO 800
      
*-- Get Neighbor Block.
      
 200  CONTINUE
      
*-- Set Parent Block Position to IDPRNT 
      IDPRNT = INT(RHDAT( 1, IBL2 ))
      
*-- Look for Neighbor Blocks.
      IPHI   = MOD( IDPRNT, 1000 )
      ITH    = INT( IDPRNT/1000 )
      NB(1)  = IDPRNT + 1000
      IF( IPHI .EQ. NMXPHI ) THEN
         NB(2) = IDPRNT - NMXPHI + 1
      ELSE
         NB(2) = IDPRNT + 1
      ENDIF
      IF( IPHI .EQ. 1 ) THEN
         NB(3) = IDPRNT - 1 + NMXPHI
      ELSE
         NB(3) = IDPRNT - 1
      ENDIF
      NB(4) = IDPRNT - 1000
      
*-- Get # of Neighbor Counter : NBTOT
      NBTOT = 0
      DO 210 IB = 1, 4
         INB = NB(IB)
         
*-- Check Neighbor Counter existance
         IF( INB .LE. 0 .OR. INB .GE. (NMXTHA+1)*1000 ) GO TO 210
         DO 220 I = INEXT, NH
            IF( INT(RHDAT(1,I)) .EQ. INB ) THEN
               NBTOT = NBTOT + 1
               JLST(NBTOT) = I
            ENDIF
 220     CONTINUE
 210  CONTINUE
      
      IF( NBTOT .LE. 0 ) GO TO 700
      
*-- Sort JLST According to ADC Count.
      CALL SORTZV(JLST,ISRT,NBTOT,-1,0,0)
      DO 230 I = 1, NBTOT
         ILST(I) = JLST(ISRT(I))
 230  CONTINUE
      
*--   ==========================================
*--    CALL BCNBLS( IDPRNT, NEXT, ILST, IFLAG )
*--   ==========================================
      
      RCUT3 = CLSPAR(4)*RHDAT( 2, IBL2 )
      
*-- Look for Daughters which .....
      
      DO 600 I = 1, NBTOT
         IBL3    = ILST(I)
         IF (IBL3.LT.INEXT) GO TO 600
         IF (REAL( RHDAT( 2, IBL2 ) ).GT.RCUT1 ) GO TO 300
         IF (REAL( RHDAT( 2, IBL3 ) ).LT.RCUT2 .AND.
     &       REAL( RHDAT( 2, IBL3 ) ).LT.RCUT3 ) GO TO 300
         GO TO 600
 300     CONTINUE
         
         LWORK1  = RHDAT( 1, IBL3 )
         LWORK2  = RHDAT( 2, IBL3 )
         LWORK3  = RHDAT( 3, IBL3 )
         IWORK   = INDX (    IBL3 )
         IF( IBL3.EQ.INEXT ) GO TO 500
         DO 400 J = 1, IBL3 - INEXT + 1
            JJ = IBL3 - J
            RHDAT( 1, JJ + 1 ) = RHDAT( 1, JJ )
            RHDAT( 2, JJ + 1 ) = RHDAT( 2, JJ )
            RHDAT( 3, JJ + 1 ) = RHDAT( 3, JJ )
            INDX (    JJ + 1 ) = INDX (    JJ )
 400     CONTINUE
         RHDAT( 1, INEXT ) = LWORK1
         RHDAT( 2, INEXT ) = LWORK2
         RHDAT( 3, INEXT ) = LWORK3
         INDX (    INEXT ) = IWORK
         
 500     CONTINUE
         INEXT = INEXT + 1
         
 600  CONTINUE

      IF( INEXT.GT.KEND ) GO TO 800
      
*-- Search for Grand-Daughter
 700  CONTINUE 
      IBL2 = IBL2 + 1
      IF( IBL2.LT.INEXT ) GO TO 200
      
*-- Search for Next Cluster
      NCLSPT( 2, NUMCLS ) = INEXT - 1
      IBL1 = INEXT
      GO TO 100
      
*-- All Cluster Found
 800  CONTINUE 
      NCLSPT( 2, NUMCLS ) = INEXT - 1
      GO TO 1000
      
 850  CONTINUE 
      NUMCLS = NUMCLS - 1
      GO TO 1000
      
*-- ===== Too many clusters
 900  CONTINUE 
      CALL MSGOUT('BCCLSR','Too many clusters found',NUMCLS)
      NUMCLS = NUMCLS - 1
      
 1000 CONTINUE
      RETURN
      END

