C   15/11/92 211150502  MEMBER NAME  SMRSHW   (S)        M  FORTRAN
      SUBROUTINE BCSHOW(ITYP,ESUM,XY,EPRT)
*.
*...BCSHOW
*.     Calculate shower energy deposit in a rectangular cell,
*.     (X_min,X_max)*(Y_min,Y_max), with the shower center at (0,0).
*.
*. INPUT     :
*.        ITYP     : (I*4) :  (1,2) = (EM,HD).
*.        ESUM     : (R*4) :  shower total energy.
*.        XY(1,1)  : (R*4) :  X_min.
*.          (2,1)  : (R*4) :  X_max.
*.          (1,2)  : (R*4) :  Y_min.
*.          (2,2)  : (R*4) :  Y_max.
*.
*. OUTPUT    :
*.        EPRT     : (R*4) :  energy deposit in the cell.
*.
*. CALLS     : 
*. CALLED    : BCHITS
*.      
*. AUTHOR    : K.Fujii
*. CREATED   : 22-May-1990
*.
*. MODIFICATION LOG
*.  06-Nov-1998 : J.Kanzaki
*.          : Converted from FJIM to JIM
*.
*.**********************************************************************
      IMPLICIT NONE

      INTEGER  ITYP
      REAL ESUM, XY(2,2), EPRT

      REAL ANM(2,2), ALM(2,2), A(2), AL(2)

      INTEGER NCALL
      DATA NCALL /0/
      SAVE NCALL
      
      REAL PX,PRT,X,AX
      INTEGER IS,IX
      INTEGER I,J,K

C========< Entry Point >================================================
C
C--
C  Initialize a lateral shower shape function:
C     f(x) = ANM(1,i)*EXP(-!x!/ALM(1,i)) + ANM(2,i)*EXP(-!x!/ALM(2,i))
C                   core part                 peripheral part
C  where i = (1,2) = (EM,HD).
C--

      IF (NCALL.EQ.0) THEN
         NCALL = 1
         ANM(1,1) = 0.14/1.263/2
         ANM(2,1) = 1.00/1.263/2
         ALM(1,1) = 0.45
         ALM(2,1) = 1.20
         ANM(1,2) = 2.00/11.40/2
         ANM(2,2) = 1.00/11.40/2
         ALM(1,2) = 2.20
         ALM(2,2) = 7.00
      ENDIF
C--
C  Decide calorimeter type.
C--
      IF (ITYP.NE.1 .AND. ITYP.NE.2) THEN
         CALL MSGOUT('SMRSHW','Illegal calorimeter type',ITYP)
         STOP
      ENDIF
      AL(1) = ALM(1,ITYP)
      AL(2) = ALM(2,ITYP)
      A(1)  = AL(1)*ANM(1,ITYP)
      A(2)  = AL(2)*ANM(2,ITYP)
C--
C  Integrate f(x)*f(y) over the cell.
C--
      PRT = 1
      DO 1000 K = 1, 2
      PX = 0
      IS = 1
      DO 100 J = 1, 2
      X  = XY(J,K)
      AX = ABS(X)
      IS = IS*(-1)
      IX = IS*SIGN(1.1,X)
      DO 10 I = 1, 2
      PX = PX + IX*A(I)*(1-EXP(-AX/AL(I)))
   10 CONTINUE
  100 CONTINUE
      PRT = PRT*PX
 1000 CONTINUE
C--
C  Calculate EPRT.
C--
      EPRT = ESUM*PRT
C--
C  That's it.
C--
      RETURN
      END
