      SUBROUTINE BCSPAR
*.
*...BCSPAR   Saves Barrel Calorimeter Parameters 
*.                         into ZBS Bank and BCGEO commom.
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : 
*. CALLS     : 
*. CALLED    : JMSAVP
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Nakamura Isamu
*. VERSION   : 1.00
*. CREATED   : 18-Apr-1995
*. LAST MOD  : 23-May-1995
*.
*. MODIFICATION LOG
*.  23-May-1995 : Nakamura.I change Bank Structure
*.  21-Apr-1995 : Nakamura Isamu
*.                Position and Size Information is added
*.  06-Mar-1996 : K.Ishii
*.                Modified position information
*.  27-Mar-1996 : K.Ishii
*.                debug
*.  26-Feb-1999 : K.Ishii
*.                Changed filled parameters
*.
*.
*.*********************************************************************
*.                                                                    *
*.     +-------------------------+                                    *
*.     | Bank Name  :  MCBCPAR   |                                    *
*.     +-------------------------+                                    *
*.                                                                    *
*.     Contents    : Information of Barrel Calorimeter Geometrical    *
*.                   Parameters are stored.                           *
*.     Record Type : 1-dim. fixed length segment                      *
*.     Version     : Release 1.0  1995-Apr-18                         *
*.                                                                    *
*.*********************************************************************
*.
*.(1) Contents in the Master Segment
*.----------------------------------
*.
*.----------------------------------------------------------------------
*.  DATA                            CONTENTS
*.----------------------------------------------------------------------
*. IDATA(1)   FLAG to select calorimeter type (See bcsub?)
*. IDATA(2)   # of Towrs in Theta Direction
*. IDATA(3)   # of Towrs in Phi Direction
*.  DATA(4)   BCAL Z length
*.  DATA(5)   Half Z length (inside)
*.  DATA(6)   Thickness of EM Scintillator Plate.
*.  DATA(7)   Thickness of HAD Scintillator Plate.
*.  DATA(8)   Depth of a Calorimeter Towor. 
*.  DATA(9)   Size of Calorimeter Towor. 
*.  DATA(10)  Depth of EM part
*.  DATA(11)  Tickness of WLS Plate.
*.  DATA(12)  Tilt Angle of Tower in Phi axis.
*.  DATA(13)  BCAL inner radius
*.
*.(Note)
*.   Master segment = segment # 0
*.
*.
*.(2) Contents in the Data Segment
*.--------------------------------
*.
*.----------------------------------------------------------------------
*.  DATA                            CONTENTS
*.----------------------------------------------------------------------
*.  DATA(1)   Position of Calorimeter Cell.
*.
*.(Note)
*.     # of Segment  6
*. Subsegment #   TYPE   # DATA        Comment
*.--------------------------------------------------------------
*.       1          R    2*NTHE     EM Cal PHI start position (rad)
*.       2          R    2*NTHE     EM Cal THETA (rad) 
*.       3          R    2*NTHE     EM Cal DISTANCE (cm)
*.       4          R    NTHE       HAD Cal PHI start position (rad)
*.       5          R    NTHE       HAD Cal THETA (rad) 
*.       6          R    NTHE       HAD Cal DISTANCE (cm)
*.  --------------------------------------------------------------------
*. Total # of segment = 6
*.
*.**********************************************************************
      IMPLICIT NONE

#include "geant321/gcvolu.inc"
#include "geant321/gconst.inc"
#include "jmdet.inc"
#include "bcgeo.inc"
#include "bchit.inc"

      INTEGER NPDATA
      PARAMETER (NPDATA=13)
      INTEGER IDATA(NPDATA)
      REAL    RDATA(NPDATA)
      EQUIVALENCE (RDATA(1),IDATA(1))
      INTEGER LNAM(15),LNUM(15)
      CHARACTER*4 BCCHDT(5)
      REAL XME1(3),XME2(3),XMH(3),XDET(3),XDH(3)
      REAL PHIH(100),THEH(100),RADH(100),DISH(100)
      REAL PHIE(2,100),THEE(2,100),RADE(2,100),DISE(2,100)
      DATA BCCHDT /'MAST','BCMA','BCP ','BL00','BA00'/
      DATA LNUM   /  15*1/

      REAL ORGX,ORGY
      REAL CORX,CORY
      INTEGER ISEG
      INTEGER IER
      INTEGER I,J,K


      IF (ISIMJD(IDETBC).EQ.ISKPJD) RETURN

      DO 10 I=1,NDTHBC
         J = 1

*-- Set the Geometric Parameters into GCVOLU Common Block.
         NLEVEL    = 0
         LNUM(3)   = J
         LNUM(4)   = I
         LNUM(5)   = I
         BCCHDT(4) = VNLBC(I)
         BCCHDT(5) = VNABC(I)
         DO 30 K=1,5
            CALL UCTOH(BCCHDT(K),LNAM(K),4,4)
 30      CONTINUE
         CALL GLVOLU(5,LNAM,LNUM,IER)

*-- Get the Calorimeter Position of HAD Cal
         XDH(1) = 0.
         XDH(2) = 0.
         XDH(3) = DEMBC/2.
         CALL GDTOM(XDH,XMH,1)

*-- Get the Calorimeter Position of EM Cal
         ORGX = -1.*(SIZBC(2,I)/2.+(SIZBC(3,I)-SIZBC(2,I))/2./4.)
         CORX = -1.*(SIZBC(5,I)/2.+(SIZBC(6,I)-SIZBC(5,I))/2./4.)
         ORGY = -1.*SIZBC(1,I)/2.
         CORY = -1.*(SIZBC(4,I)-SIZBC(1,I))/2.
         XDET(1) = ORGX+(CORX-ORGX)*DEMBC/BLBC/2.
         XDET(2) = ORGY+(CORY-ORGY)*DEMBC/BLBC/2.
         XDET(3) = -(BLBC-BSBC)/2.
         CALL GDTOM(XDET,XME1,1)

         ORGX = -1.*(SIZBC(2,I)/2.+(SIZBC(3,I)-SIZBC(2,I))/2.*3./4.)
         CORX = -1.*(SIZBC(5,I)/2.+(SIZBC(6,I)-SIZBC(5,I))/2.*3./4.)
         ORGY = SIZBC(1,I)/2.
         CORY = (SIZBC(4,I)-SIZBC(1,I))/2.
         XDET(1) = ORGX+(CORX-ORGX)*DEMBC/BLBC/2.
         XDET(2) = ORGY+(CORY-ORGY)*DEMBC/BLBC/2.
         CALL GDTOM(XDET,XME2,1)
         
*-- Arrange the Parameter into Some Values
         PHIH(I)=ATAN2(XMH(2),XMH(1))
         RADH(I)=SQRT(XMH(1)**2+XMH(2)**2)
         THEH(I)=ATAN2(RADH(I),XMH(3))
         DISH(I)=SQRT(XMH(1)**2+XMH(2)**2+XMH(3)**2)
         PHIE(1,I)=ATAN2(XME1(2),XME1(1))
         RADE(1,I)=SQRT(XME1(1)**2+XME1(2)**2)
         THEE(1,I)=ATAN2(RADE(1,I),XME1(3))
         DISE(1,I)=SQRT(XME1(1)**2+XME1(2)**2+XME1(3)**2)
         PHIE(2,I)=ATAN2(XME2(2),XME2(1))
         RADE(2,I)=SQRT(XME2(1)**2+XME2(2)**2)
         THEE(2,I)=ATAN2(RADE(2,I),XME2(3))
         DISE(2,I)=SQRT(XME2(1)**2+XME2(2)**2+XME2(3)**2)
 10   CONTINUE

*-- Create ZBS Bank MCBCPAR
      CALL KZBCR0('MCBCPAR',IER)

*-- Store Data to given Segment of Bank MCBCPAR
      IDATA(1)  = SWITBC(1)
      IDATA(2)  = NDTHBC
      IDATA(3)  = NDPHBC
      RDATA(4)  = ZLBC
      RDATA(5)  = ZIBC
      RDATA(6)  = TSCEBC
      RDATA(7)  = TSCHBC
      RDATA(8)  = BLBC
      RDATA(9)  = BSBC
      RDATA(10) = DEMBC
      RDATA(11) = TLGBC
      RDATA(12) = PTLTBC
      RDATA(13) = RINBC

*-- Fill the Master Segment
      CALL KZREP0('MCBCPAR',0,'I',3,IDATA(1))
      CALL KZREP0('MCBCPAR',0,'F',10,RDATA(4))

*-- Fill the Secondery Segments

*-- (1) EM Cal PHI start position (rad)
      ISEG = 0
      DO 50 I=1,NDTHBC
         ISEG           = ISEG + 2
         RDATA(1)       = PHIE(1,I)
         GPE1BC(ISEG-1) = PHIE(1,I)
         CALL  KZREP0('MCBCPAR',1,'F',1,IDATA)
         RDATA(1)       = PHIE(2,I)
         GPE1BC(ISEG)   = PHIE(2,I)
         CALL  KZREP0('MCBCPAR',1,'F',1,IDATA)
 50   CONTINUE

*-- (2) EM Cal THETA (rad) 
      ISEG = 0
      DO 51 I=1,NDTHBC
         ISEG           = ISEG + 2
         RDATA(1)       = THEE(1,I)
         GPE2BC(ISEG-1) = THEE(1,I)
         CALL  KZREP0('MCBCPAR',2,'F',1,IDATA)
         RDATA(1)       = THEE(2,I)
         GPE2BC(ISEG)   = THEE(2,I)
         CALL  KZREP0('MCBCPAR',2,'F',1,IDATA)
 51   CONTINUE

*-- (3) EM Cal DISTANCE (cm)
      ISEG = 0
      DO 52 I=1,NDTHBC
         ISEG           = ISEG +2
         RDATA(1)       = DISE(1,I)
         GPE3BC(ISEG-1) = DISE(1,I)
         CALL  KZREP0('MCBCPAR',3,'F',1,IDATA)
         RDATA(1)       = DISE(2,I)
         GPE3BC(ISEG)   = DISE(2,I)
         CALL  KZREP0('MCBCPAR',3,'F',1,IDATA)
 52   CONTINUE

*-- (4) HAD Cal PHI start position (rad)
      ISEG = 0
      DO 53 I=1,NDTHBC
         ISEG           = ISEG + 1
         RDATA(1)       = PHIH(I)
         GPH1BC(ISEG)   = PHIH(I)
         CALL  KZREP0('MCBCPAR',4,'F',1,IDATA)
 53   CONTINUE

*-- (5) HAD Cal THETA (rad)
      ISEG = 0
      DO 54 I=1,NDTHBC
         ISEG           = ISEG + 1
         RDATA(1)       = THEH(I)
         GPH2BC(ISEG)   = THEH(I)
         CALL  KZREP0('MCBCPAR',5,'F',1,IDATA)
 54   CONTINUE

*-- (6) HAD Cal DISTANCE (cm)
      ISEG = 0
      DO 55 I=1,NDTHBC
         ISEG           = ISEG + 1 
         RDATA(1)       = DISH(I)
         GPH3BC(ISEG)   = DISH(I)
         CALL  KZREP0('MCBCPAR',6,'F',1,IDATA)
 55   CONTINUE

      RETURN
      END
       
