      SUBROUTINE IRFFRD
*.
*...IRFFRD   Read IR parameters as QC-Magnet geometry data
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : GEFILE
*. CALLS     : FFSET  FFKEY  FFGO 
*. CALLED    : UGINIT
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Yoshiaki Fujii
*. VERSION   : 1.00
*. CREATED   : 28-Aug-1996
*.
*. MODIFICATION LOG
*.  xx-Oct-1999 T.Tauchi
*.              change in BPGEO
*.
*.**********************************************************************
      IMPLICIT NONE

*-- Common for user
#include "jmdet.inc"
#include "irgeo.inc"
#include "amgeo.inc"

*
      CRSAIR = -0.004
*
      QXY1QC(1) = 0.474
      QXY1QC(2) = 0.474
      QXY2QC(1) = 0.85
      QXY2QC(2) = 0.264
      QXY3QC(1) = 1.0
      QXY3QC(2) = 0.264
      QXY4QC(1) = 2.0
      QXY4QC(2) = 1.0
      QXY5QC(1) = 4.0
      QXY5QC(2) = 1.4
      QXY6QC(1) = 5.3
      QXY6QC(2) = 2.6
      QXY7QC(1) = 9.5
      QXY7QC(2) = 2.6
      QXY8QC(1) = 11.0
      QXY8QC(2) = 5.0
*
      QC0ZQC(1) = 199.
      QC0ZQC(2) = 200.
*
      QC1ZQC(1) = 200.
      QC1ZQC(2) = 420.
*
      QC2ZQC(1) = 440.
      QC2ZQC(2) = 840.
*
      COIXQC(1) = 5.300
      COIXQC(2) = 9.000
      COIYQC(1) = 0.500
      COIYQC(2) = 2.100

*-- Active Mask Geometry Parameters
*--   Theta 1 of Mask
      THETMK(1) = 0.150
*--   Theta 2 of Mask
      THETMK(2) = 0.200
*--   Inner most radius of Mask
      R1INMK = 4.5
*--   Outer most radius of Mask
      R2OUMK = 40.72
*--   Largest Z of Mask 
      Z3MK = 840.

*-- Geometry parameter for support tube
*--   inner radius
      RRZST(1) = 40.72
*--   outer radius
      RRZST(2) = 41.72
*--   half z-length
      RRZST(3) = 1000.

*-- Geometry parameter for compensation magnet
*--   inner radius
      RRZZCS(1) = 15.
*--   outer radius
      RRZZCS(2) = 27.
*--   Z1
      RRZZCS(3) = 180.
*--   Z2
      RRZZCS(4) = 840.
*--   Thickness of active layer
      TSILAM = 0.02
*--                support structure
      TSUPAM = 0.03
*--   # of layers in active mask
      NLAYAM = 4
*--   depth of 1st active layer
      DEPTAM(1) = 0.5
*--            2nd 
      DEPTAM(2) = 1.5
*--            3rd
      DEPTAM(3) = 2.5
*--            4th
      DEPTAM(4) = 3.5
*--    # phi division of mask 
      NPHIAM = 32
*--    size of active layer in r-direction
      DRAM = 0.2

*-- geometry parameter of beam pipe      
      RINBP(1) = 1.8
      RINBP(2) = 7.3
      RINBP(3) = 1.8
      THIKBP(1) = 0.05
      THIKBP(2) = 0.2
      THIKBP(3) = 5.7
      THIKBP(4) = 0.2

      ZPOSBP(1) = 13.233
      ZPOSBP(2) = 49.624
      ZPOSBP(3) = 175.
      ZPOSBP(4) = 175.2
      ZPOSBP(5) = 200.

      IF (LUNMJD(IDETIR).LE.0) RETURN

*-- Set optional values
      CALL FFSET('LINP', LUNMJD(IDETIR))

*-- Define private data records

*-- Crossing Angle in mrad
      CALL FFKEY('CROS', CRSAIR , 1, 'REAL')

*-- QXYn = (x,y) of n-th point of QC cross section.
*-- QCnZ = z1,z2 of QCn magnet faces. 
      CALL FFKEY('QCXY', QXYNQC ,16, 'REAL')

      CALL FFKEY('QC0Z', QC0ZQC , 2, 'REAL')
      CALL FFKEY('QC1Z', QC1ZQC , 2, 'REAL')
      CALL FFKEY('QC2Z', QC2ZQC , 2, 'REAL')

      CALL FFKEY('QCLX', COIXQC , 2, 'REAL')
      CALL FFKEY('QCLY', COIYQC , 2, 'REAL')

*-- Mask Geometry Parameters
      CALL FFKEY('MKTH', THETMK , 2, 'REAL')
      CALL FFKEY('MKR1', R1INMK , 1, 'REAL')
      CALL FFKEY('MKR2', R2OUMK , 1, 'REAL')
      CALL FFKEY('MKZ3', Z3MK   , 1, 'REAL')

*-- Beam Pipe Geometry Parameters
      CALL FFKEY('BPRI',  RINBP , 2, 'REAL')
      CALL FFKEY('BPTK', THIKBP , 2, 'REAL')
      CALL FFKEY('BPZP', ZPOSBP , 3, 'REAL')

*-- Active Mask Geometry Parameters
      CALL FFKEY('AMSI', TSILAM , 1, 'REAL')
      CALL FFKEY('AMSU', TSUPAM , 1, 'REAL')
      CALL FFKEY('AMLY', NLAYAM , 1, 'INTE')
      CALL FFKEY('AMDE', DEPTAM , 4, 'REAL')
      CALL FFKEY('AMPH', NPHIAM , 1, 'INTE')
      CALL FFKEY('AMDR', DRAM   , 1, 'REAL')

*-- Support Tube Geometry Parameters
      CALL FFKEY('STUB', RRZST, 3, 'REAL')

*-- Compensation Solenoid Geometry Parameters
      CALL FFKEY('CSRZ',RRZZCS, 4, 'REAL')

*-- Read data cards
      CALL FFGO

  999 CONTINUE
      RETURN
      END
