      SUBROUTINE KIMULT
C
C     Similar to KISING, but generate plural particles.  
C     This routine is activated by specifying 6 for KINE(1) in geant.dat. 
C     The meaning of keyword KINE(2) - KINE(11) are
C         KINE(2) = PKINE(1) = Energy of Generated Particles
C         KINE(3) = PKINE(2) = LUND Particle Code for Generaterd Particle
C         KINE(4) = PKINE(3) = Number of Generated Particle per event
C         KINE(5) = PKINE(4) = THETA-min for Generation
C         KINE(6) = PKINE(5) = THETA-max for Generation
C         KINE(7) = PKINE(6) = PHAI-min  for Generation
C         KINE(8) = PKINE(7) = PHAI-max  for Generation
C         KINE(9) = PKINE(8) = X-coordinate for Generation Vertex
C         KINE(10)= PKINE(9) = Y-coordinate for Generation Vertex 
C         KINE(11)= PKINE(10)= Z-coordinate for Generation Vertex
C     
*.
*...GUKINE   GEANT3 user routine
*.           Generates kinematics of primary tracks
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : GCFLAG GCONST GCKINE
*. CALLS     : GSVERT GFPART GSKINE GPKINE GRNDM JLP2GP
*. CALLED    : GTRIG
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Yoshiaki Fujii
*. VERSION   : 1.00
*. CREATED   : 28-Aug-1996
*. LAST MOD  :  1-Oct.1996
*.
*. MODIFICATION LOG
*.      1-Oct-1996, Y.Fujii ; Re-defined KINE card meaning to include 
*.                            particle vertex position.
*.
*.**********************************************************************
      IMPLICIT NONE

*--   Common for GEANT
#include "geant321/gcflag.inc"
#include "geant321/gconst.inc"
#include "geant321/gckine.inc"

#include "hepevt.inc"

      REAL UB(10),RNDM(2)
      INTEGER NWB
      CHARACTER*20 NAMPRT

      REAL THMIN,THMAX
      REAL FIMIN,FIMAX
      INTEGER IPN,IPNGEA
      REAL THETA,PHI
      INTEGER IERR,I

*-- Pcode and Nevent
      IPN =NINT(PKINE(2))
      NHEP=NINT(PKINE(3))

      IF (NHEP.EQ.0) NHEP = 1

*-- theta and phai
      THMIN = PKINE(4)*DEGRAD
      THMAX = PKINE(5)*DEGRAD

      FIMIN = PKINE(6)*DEGRAD
      FIMAX = PKINE(7)*DEGRAD

      NEVHEP   =1

*-- Fill LEHEP variables which are independent of particle #.
      DO 100 I=1,NHEP
         IDHEP(I)=IPN
         ISTHEP(I)=1
         JMOHEP(1,I)=0
         JMOHEP(2,I)=0
         JDAHEP(1,I)=0
         JDAHEP(2,I)=0
         VHEP(1,I)= PKINE(8)
         VHEP(2,I)= PKINE(9)
         VHEP(3,I)= PKINE(10)
         VHEP(4,I)=0.0
  100 CONTINUE      

*-- Loop over Number of Particles

      DO 200 I=1,NHEP

*-- Generate random number for Theta and Phai
         CALL GRNDM(RNDM,2)
         THETA = (THMAX-THMIN)*RNDM(1) + THMIN
         PHI   = (FIMAX-FIMIN)*RNDM(2) + FIMIN

*-- Get particle mass from the data structure JPART
         CALL JLP2GP(IPN,IPNGEA,IERR)
         CALL GFPART(IPNGEA,NAMPRT,ITRTYP,AMASS,CHARGE,TLIFE,UB,NWB)

*-- Define momentum
         PHEP(1,I) = PKINE(1)*SIN(THETA)*COS(PHI)
         PHEP(2,I) = PKINE(1)*SIN(THETA)*SIN(PHI)
         PHEP(3,I) = PKINE(1)*COS(THETA)
         PHEP(4,I) = SQRT( PHEP(1,I)**2 + PHEP(2,I)**2 
     &                   + PHEP(3,I)**2 + AMASS**2 )
         PHEP(5,I) = AMASS

  200 CONTINUE         

      RETURN
      END
