      SUBROUTINE LMGEOM
*.
*...LMGEOM   Geomtry of Luminosity monitor and masks around it.
*.           all geometry can be CONE.
*.                  z1   - z2   - z3       inputs are ; theta-in
*.       theta-in   r1in - r2in - r2in                  theta-out 
*.       theta-out  r1out- r2out- r2out                 r1in,r2out,z3
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : UCTMED
*. CALLS     : 
*. CALLED    : IRGEOM
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*. AUTHOR    : T.Tauchi
*. VERSION   : 0.00
*. CREATED   : 4-Feb-1997
*. LAST MOD  : 
*.
*. MODIFICATION LOG
*. 25-Feb-1999 Nakamura use copy for backword counter
*. 12-Jan-1998 Y.Fujii ; Added jmmat.inc and jmdet.inc
*.                       Added ISIMJD(IDETBM) judgement
*.                       Added  GSTMED for W and CFRP. 
*.
*.**********************************************************************
      IMPLICIT NONE

#include "jmmed.inc"
#include "jmdet.inc"
#include "jmmat.inc"
#include "jmrot.inc"
#include "lmgeo.inc"
#include "lmhit.inc"
      
    
*-- Tracking media parameters for LUMIS
      REAL STEMXS,EPSILS,STMINS
      PARAMETER ( STEMXS=0.01 )
      PARAMETER ( EPSILS=0.001)
      PARAMETER ( STMINS=0.001)

      INTEGER ILUMI,ICFRP,ILUMIS
      REAL R1INLM,R1OULM
      REAL R2INLM,R2OULM
      REAL BMZFW
C      REAL BMZFW,BMZBW
      
      INTEGER IVOLU,IROT
      INTEGER IDET,ISET
      INTEGER NBITSV(NDSVLM)
      DATA NBITSV/ 2, 6, 6/
      CHARACTER*4 CHNM(NDSVLM)
      DATA CHNM/'LMC ','LMCP','LMCR'/

*-- Hit Bank Parameter
      CHARACTER*4 CHNAMH(NDHTLM)
      INTEGER NBITSH(NDHTLM)
      REAL ORIG(NDHTLM),FACT(NDHTLM)
      DATA CHNAMH /
     &     'PCOD','X   ','Y   ','Z   ','P   ',
     &     'Px  ','Py  ','Pz  ','ELOS'/
      DATA NBITSH/
     &     32    ,32    ,32    ,32    ,32    ,
     &     32    ,32    ,32    ,32/
      DATA ORIG/
     &     30000.,200.  ,200.  ,200.  ,0.    ,
     &     500.  ,500.  ,500.  ,0./
      DATA FACT/
     &     1.    ,1.E6  ,1.E6  ,1.E6  ,1.E6  ,
     &     1.E6  ,1.E6  ,1.E6  ,1.E6/
      REAL   PAR(5)

      IF (ISIMJD(IDETLM).EQ.ISKPJD) GOTO 999

      ILUMI = NMEDJM*IDETLM+1
      CALL GSTMED(ILUMI,'LUMI', IMTUNG, NOSENS, IFIELD, FIELDM,
     &     TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,UBUF,NWBUF )
      ICFRP = NMEDJM*IDETLM+2
      CALL GSTMED(ICFRP, 'CFRP', IMCARB, NOSENS, IFIELD, FIELDM,
     &     TMAXFD, STEMAX, DEEMAX, EPSIL, STMIN, UBUF, NWBUF)
      ILUMIS = NMEDJM*IDETLM+3
      CALL GSTMED(ILUMIS,'LUMIS', IMTUNG, ISENS, IFIELD, FIELDM,
     &     TMAXFD, STEMXS, DEEMAX, EPSILS,STMINS,UBUF,NWBUF )

*-- Define Geometry for Luminosity Monitor.
      R1INLM = ZLM(1,1)*TAN(THETLM(1,1))
      R1OULM = ZLM(1,1)*TAN(THETLM(2,1))
      R2INLM = ZLM(2,1)*TAN(THETLM(1,1))
      R2OULM = ZLM(2,1)*TAN(THETLM(2,1))

*--   +Z
      PAR(1) = ( ZLM(2,1) - ZLM(1,1)) / 2.0
      PAR(2) = R1INLM
      PAR(3) = R1OULM
      PAR(4) = R2INLM
      PAR(5) = R2OULM
      BMZFW  = ( ZLM(2,1) + ZLM(1,1)) / 2.0
      CALL GSVOLU('LMC ','CONE',ILUMIS,PAR,5,IVOLU)
      CALL GSPOS('LMC ',1,'MAST',0,0,BMZFW,0,'ONLY')
*-- rotation matrix for backward counter
      CALL GSPOS('LMC ',2,'MAST',0,0,-BMZFW,IRMIRR,'ONLY')
*
*-- divide in phi
      CALL GSDVN('LMCP','LMC ',NPDLM, 2)
*
*-- divide in R
      CALL GSDVN('LMCR','LMCP',NRDLM, 1)
*
*-- define HIT bank
      CALL GSDET('LM  ','LMC ',NDSVLM,CHNM,NBITSV,
     &     NDETJM*IDETLM+1,NHMXLM,0,ISET,IDET)
*-- Define hit parameters for detector of set BM.
      CALL GSDETH('LM  ','LMC ',NDHTLM,CHNAMH,NBITSH,ORIG,FACT)

      CALL GPSETS('LM  ','LMC ')

*-- Define Geometry for LM-support.
      R1INLM = ZLM(1,2)*TAN(THETLM(1,2))
      R1OULM = ZLM(1,2)*TAN(THETLM(2,2))
      R2INLM = ZLM(2,2)*TAN(THETLM(1,2))
      R2OULM = ZLM(2,2)*TAN(THETLM(2,2))

*-- +Z
      PAR(1) = ( ZLM(2,2) - ZLM(1,2) ) / 2.0
      PAR(2) = R1INLM
      PAR(3) = R1OULM
      PAR(4) = R2INLM
      PAR(5) = R2OULM
      BMZFW  = ( ZLM(2,2)+ ZLM(1,2)) / 2.0
      CALL GSVOLU('LMS ','CONE',ILUMI,PAR,5,IVOLU)
      CALL GSPOS('LMS ',1,'MAST',0,0,BMZFW,0,'ONLY')
      CALL GSPOS('LMS ',2,'MAST',0,0,-BMZFW,IRMIRR,'ONLY')

*-- Define Geometry for carbon mask in front of Luminosity Monitor.
      R1INLM = ZLM(1,4)*TAN(THETLM(1,4))
      R1OULM = ZLM(1,4)*TAN(THETLM(2,4))
      R2INLM = ZLM(2,4)*TAN(THETLM(1,4))
      R2OULM = ZLM(2,4)*TAN(THETLM(2,4))
*-- +Z
      PAR(1) = ( ZLM(2,4) - ZLM(1,4) ) / 2.0
*
*-- If thickness <= 0. don't define this volume
      IF (PAR(1).GT.0.) THEN
         PAR(2) = R1INLM
         PAR(3) = R1OULM
         PAR(4) = R2INLM
         PAR(5) = R2OULM
         BMZFW  = ( ZLM(2,4)+ ZLM(1,4)) / 2.0
         CALL GSVOLU('LMF ','CONE',ICFRP,PAR,5,IVOLU)
         CALL GSPOS('LMF ',1,'MAST',0,0,BMZFW,0,'ONLY')
         CALL GSPOS('LMF ',2,'MAST',0,0,-BMZFW,IRMIRR,'ONLY')
      ENDIF

*-- Define Geometry for inner mask (inside of LM).
*-- Its material is Carbon.
*-- if thickness <= 0. don't define this volume
      IF (ZLM(2,3).NE.ZLM(1,3) ) THEN
         IF (THETLM(1,3).GT.0.0) THEN
            R1INLM = ZLM(1,3)*TAN(THETLM(1,3))
            R2INLM = ZLM(2,3)*TAN(THETLM(1,3))
         ELSE
            R1INLM = RMKLM
            R2INLM = RMKLM
         ENDIF
         R1OULM = ZLM(1,3)*TAN(THETLM(2,3))
         R2OULM = ZLM(2,3)*TAN(THETLM(2,3))
*-- +Z
         PAR(1) = ( ZLM(2,3) - ZLM(1,3) ) / 2.0
         PAR(2) = R1INLM
         PAR(3) = R1OULM
         PAR(4) = R2INLM
         PAR(5) = R2OULM
         BMZFW  = ( ZLM(2,3) + ZLM(1,3) ) / 2.0
         CALL GSVOLU('LMCM','CONE',ICFRP,PAR,5,IVOLU)
         CALL GSPOS('LMCM',1,'MAST',0,0,BMZFW,0,'ONLY')
         CALL GSPOS('LMCM',2,'MAST',0,0,-BMZFW,IRMIRR,'ONLY')
      ENDIF

 999  CONTINUE 
      RETURN
      END
      
