      SUBROUTINE MKGEOM
*.
*...BCGEOM   Geomtry of the Masks at IR. 
*.           Mask =  Cone + Tube
*.                  z1   - z2   - z3       inputs are ; theta-in
*.       theta-in   r1in - r2in - r2in                  theta-out 
*.       theta-out  r1out- r2out- r2out                 r1in,r2out,z3
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : UCTMED
*. CALLS     : 
*. CALLED    : IRGEOM
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*. AUTHOR    : Yoshiaki Fujii
*. VERSION   : 1.00
*. CREATED   : 29-Aug-1996
*. LAST MOD  : 
*.
*. MODIFICATION LOG
*.
*.**********************************************************************
      IMPLICIT NONE

#include "jmdet.inc"
#include "jmmat.inc"
#include "jmmed.inc"
#include "jmrot.inc"
#include "irgeo.inc"
#include "amgeo.inc"
#include "amhit.inc"
      
      INTEGER IMASK
      REAL   PAR(11)

      INTEGER ISILI
    
*-- Tracking media parameters for Si
      REAL STEMXS,EPSILS,STMINS
      PARAMETER ( STEMXS=0.01 )
      PARAMETER ( EPSILS=0.0001)
      PARAMETER ( STMINS=0.0001)
      
      INTEGER IG10
      
      REAL Z1MK,Z2MK
      REAL R1OUMK,R2INMK
      INTEGER IVOLU

      REAL Z0,Z1,Z2
      REAL RIN0,ROU0,RIN1,ROU1,RIN2,ROU2
      CHARACTER*4 VNLYAM,VNSIAM,VNSUAM
      CHARACTER*4 VNRDAM,VNPHAM

      CHARACTER*4 CHNMAM(NDSVAM)
      INTEGER NBITSV(NDSVAM)
      DATA NBITSV/  8,  8,  8,  8/

      CHARACTER*4 CHNAMH(NDHTAM)
      INTEGER NBITSH(NDHTAM)
      REAL ORIG(NDHTAM),FACT(NDHTAM)
      DATA CHNAMH /  'PCOD','X   ','Y   ','Z   ','P   ',
     &     'Px  ','Py  ','Pz  ','ELOS'/
      DATA NBITSH/ 1*16,3*32,32,3*32,32 /
      DATA ORIG/ 30000.,3*200.,0.,3*500.,0./
      DATA FACT/ 1., 3*1.E6,1.E6, 3*1.E6,1.E6/

      REAL Z
      INTEGER IROT
      INTEGER IDET,ISET,IDTYPE

      INTEGER I

*-- Track media definition
*-- Mask (Tungsten)
      IMASK = NMEDJM*IDETIR+IDMKIR+1
      CALL GSTMED(IMASK,'MASK', IMTUNG, NOSENS, IFIELD, FIELDM,
     &     TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,UBUF,NWBUF )
*-- active volume (Silicon)
      ISILI = NMEDJM*IDETIR+IDMKIR+2
      CALL GSTMED(ISILI,'SIAM',IMSILI,ISENS,IFIELD, FIELDM,
     &     TMAXFD, STEMXS, DEEMAX, EPSILS,STMINS,UBUF, NWBUF)
*-- support structure (G10)
      IG10 = NMEDJM*IDETIR+IDMKIR+3
      CALL GSTMED(IG10,'G10AM',IMG10,NOSENS,IFIELD, FIELDM,
     &     TMAXFD, STEMXS, DEEMAX, EPSILS,STMINS,UBUF, NWBUF)

*-- Define Geometry for Conical Part.
      Z1MK   = R1INMK / TAN(THETMK(1))
      R1OUMK = Z1MK   * TAN(THETMK(2))
      Z2MK   = R2OUMK / TAN(THETMK(2))
      R2INMK = Z2MK   * TAN(THETMK(1))

      PAR(1) = 0.5*( Z2MK - Z1MK )
      PAR(2) = R1INMK
      PAR(3) = R1OUMK
      PAR(4) = R2INMK
      PAR(5) = R2OUMK
      CALL GSVOLU('MKCN','CONE',IMASK,PAR,5,IVOLU)
      CALL GSATT('MKCN','SEEN',1)
      
*-- Define Active Mask geometries
*-- loop over layers      
      DO 100 I=1,NLAYAM
         Z0 = Z1MK+DEPTAM(I)
         RIN0 = Z0 * TAN(THETMK(1))
         ROU0 = Z0 * TAN(THETMK(2))
         Z1 = Z0 + TSILAM
         RIN1 = Z1 * TAN(THETMK(1))
         ROU1 = Z1 * TAN(THETMK(2))
         Z2 = Z1 + TSUPAM
         RIN2 = Z2 * TAN(THETMK(1))
         ROU2 = Z2 * TAN(THETMK(2))
         PAR(1) = 0.5*(TSILAM+TSUPAM)
         PAR(2) = RIN0
         PAR(3) = ROU0
         PAR(4) = RIN2
         PAR(5) = ROU2

*-- define layer
         WRITE(VNLYAM,'(''AML'',I1)') I
         CALL GSVOLU(VNLYAM,'CONE',IMASK,PAR,5,IVOLU)

*-- calculate size of silicon
         PAR(1) = 0.5*TSILAM
         PAR(2) = RIN0
         PAR(3) = ROU0
         PAR(4) = RIN1
         PAR(5) = ROU1
         WRITE(VNSIAM,'(''AMS'',I1)') I
         CALL GSVOLU(VNSIAM,'CONE',ISILI,PAR,5,IVOLU)
*-- divide AML? in phi direction 
*-- name of geometry
         WRITE(VNPHAM,'(''AMP'',I1)') I
         CALL GSDVN(VNPHAM,VNSIAM,NPHIAM,2)
*-- divide AMP? in r direction 
         WRITE(VNRDAM,'(''AMR'',I1)') I
         NTHEAM(I) = NINT((ROU0-RIN0)/DRAM)
         CALL GSDVN(VNRDAM,VNPHAM,NTHEAM(I),1)
*-- put silicon on the layer
         Z = -0.5*TSUPAM
         CALL GSPOS(VNSIAM,1,VNLYAM,0.,0.,Z,0,'ONLY')
*-- setup Detector name tree for GSDET
         CHNMAM(1) = 'MKCN'
         CHNMAM(2) = VNLYAM
         CHNMAM(3) = VNPHAM
         CHNMAM(4) = VNRDAM
         IDTYPE = NDETJM*IDETAM+I
*-- Assign detector to the set AM and defines its basic params.
         CALL GSDET('AM  ',VNRDAM,NDSVAM,CHNMAM,NBITSV,IDTYPE,NHMXAM,0,
     &        ISET,IDET)
*--      Define hit parameters for detector of set BM.
         CALL GSDETH('AM  ',VNRDAM,NDHTAM,CHNAMH,NBITSH,ORIG,FACT)
*-- save detector name?
         VNAMAM(I) = VNRDAM
*-- size of support structure
         PAR(1) = 0.5*TSUPAM
         PAR(2) = RIN1
         PAR(3) = ROU1
         PAR(4) = RIN2
         PAR(5) = ROU2
         WRITE(VNSUAM,'(''AMU'',I1)') I
         CALL GSVOLU(VNSUAM,'CONE',IG10,PAR,5,IVOLU)
*-- put support on the layer
         Z = 0.5*TSILAM
         CALL GSPOS(VNSUAM,1,VNLYAM,0.,0.,Z,0,'ONLY')
*-- put the layer on the MASK
         Z = DEPTAM(I)+0.5*(TSILAM+TSUPAM)-0.5*(Z2MK-Z1MK)
         CALL GSPOS(VNLYAM,I,'MKCN',0.,0.,Z,0,'ONLY')

 100  CONTINUE

*-- Put the Conical part into Master Volume
C      IROT = NROTJM*IDETIR+10*IDMKIR+1
C      CALL GSROTM(IROT,90.,0.,90.,90.,180.,0.)
      Z   = 0.5*( Z2MK + Z1MK )
      CALL GSPOS('MKCN',1,'MAST',0.,0., Z,0,'ONLY')
      CALL GSPOS('MKCN',2,'MAST',0.,0.,-Z,IRMIRR,'ONLY')

*-- Define Tube Part
      PAR(1) = R2INMK
      PAR(2) = R2OUMK
      PAR(3) =0.5*( Z3MK - Z2MK )
      CALL GSVOLU('MKTB','TUBE',IMASK,PAR,3,IVOLU)
      Z =  0.5*( Z3MK + Z2MK )
      CALL GSPOS('MKTB',1,'MAST',0.,0.,Z,0,'ONLY')
      CALL GSPOS('MKTB',2,'MAST',0.,0.,-Z,IROT,'ONLY')

      RETURN
      END
