      SUBROUTINE SOGEOM
*.
*...BCGEOM   Geomtry of the JLC Solenoid Magnet.
*.
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : UCTMED
*. CALLS     : 
*. CALLED    : UGEOM
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*. AUTHOR    : Nakamura Isamu
*. VERSION   : 1.00
*. CREATED   : 16-June-1994
*. LAST MOD  : 
*.
*. MODIFICATION LOG
*. 17-May-1995 Y.Fujii, Fixed bugs in iron polygon.
*.
*.**********************************************************************
      IMPLICIT NONE

#include "jmmed.inc"
#include "jmmat.inc"
#include "jmdet.inc"
      
      INTEGER ICOIL,IIRONB,IIRONE

      REAL     RSOIN,RSOOUT,LSO,LSOFW,LSOMD,LSOBW,ZSOFW,ZSOMD,ZSOBW      
      REAL     FIMIN, FISPAN, RMIN, RMAX, ZB, ZF 
      REAL     RFEIN(5), RFEOUT(5)
      REAL     RECIN(2), RECOUT(2), ZEC1(6), ZEC2(6)
      INTEGER   NSIDE, NZ
      CHARACTER*4   CNAME
      CHARACTER*1   CDATA1(4)
      CHARACTER*1   CDATA2(4)
      REAL PAR(11)

      REAL RVSLOU,RVSLIN
      REAL ZECF1
      INTEGER LVSL,IFB
      INTEGER IVOLU
      INTEGER IL
      
*-- DATA FOR SOLENOID COIL GEOMETRY
*-- (THE SOLENOID IS DEVIDED INTO THREE IN Z; FORWARD, MIDDLE, BACKWARD)
      DATA       RSOIN, RSOOUT / 450.0, 460.0 /
      DATA       LSO, LSOFW, LSOMD, LSOBW / 980.0, 3 * 250.0 /
      DATA       ZSOFW, ZSOMD, ZSOBW / 330.0, 0.0, -330.0 /
      DATA       RVSLIN, RVSLOU, LVSL / 430.0, 480.0, 980.0 /

*-- DATA FOR RETURN YOKE GEOMETRY
*-- (THE YOKE IS DEVIDED INTO FIVE LAYERS IN R TO PUT MUONS INBETWEEN.)
      DATA     FIMIN, FISPAN / 22.5, 360.0 /
      DATA     RMIN, RMAX / 500.0, 700.0 /
      DATA     ZF, ZB / 500.0, -500.0 /
      DATA     RFEIN  / 500.0, 542.0, 584.0, 626.0, 668.0 /
      DATA     RFEOUT / 532.0, 574.0, 616.0, 658.0, 700.0 /
      DATA     NSIDE, NZ / 8, 2 /
      DATA     CDATA1 / 'F', 'E', 'L', ' ' /

*-- DATA FOR ENDCAP RETURN YOKE GEOMETRY.
*-- THE YOKE IS DEVIDED INTO ONE FRONT LAYER AND REAR FIVE LAYERS
*-- IN Z TO PUT MUONS INBETWEEN.
*-- NOTE THAT THE ENDCAP IRON IS DIFFERENT FROM BARREL IRON.
      DATA    RECIN  /  45.0,  50.0 /
      DATA    RECOUT / 400.0, 700.0 /
      DATA    ZEC1   / 480.0, 500.0, 560.0, 620.0, 680.0, 740.0 /
      DATA    ZEC2   / 500.0, 550.0, 610.0, 670.0, 730.0, 790.0 /
      DATA    CDATA2 / 'E', 'C', ' ', ' ' /

*-- =====  TEMPLATE FOR SOLENOID GEOMETRY  =====

      IF (ISIMJD(IDETSO).EQ.ISKPJD) RETURN

      ICOIL = NMEDJM*IDETSO+1
      CALL GSTMED(ICOIL,'COIL', IMCOIL , NOSENS, NOFILD, 0.,
     &            TMAXFD, STEMAX, DEEMAX, EPSIL, STMIN, UBUF, NWBUF)

      IIRONB = NMEDJM*IDETSO+2
      CALL GSTMED(IIRONB,'IRONBL', IMIRON, NOSENS, IFIELD, FIELDF,
     &     TMAXFD, STEMAX, DEEMAX, EPSIL, STMIN, UBUF, NWBUF )

      IIRONE = NMEDJM*IDETSO+3
      CALL GSTMED(IIRONE,'IRONEC', IMIRON, NOSENS, IFIELD, FIELDM,
     &     TMAXFD, STEMAX, DEEMAX, EPSIL, STMIN, UBUF, NWBUF )

*-- DEFINE GEOMETRY
      PAR(1) = RVSLIN
      PAR(2) = RVSLOU
      PAR(3) = LVSL/2.0
      CALL GSVOLU('VESL','TUBE',IAIRJM,PAR,3,IVOLU)
      PAR(1) = RSOIN
      PAR(2) = RSOOUT
      PAR(3) = LSO/2.0
      CALL GSVOLU('SOMA','TUBE',IAIRJM,PAR,3,IVOLU)
      PAR(1) = RSOIN
      PAR(2) = RSOOUT
      PAR(3) = LSOFW/2.0
      CALL GSVOLU('SOFW','TUBE',ICOIL,PAR,3,IVOLU)
      PAR(1) = RSOIN
      PAR(2) = RSOOUT
      PAR(3) = LSOMD/2.0
      CALL GSVOLU('SOMD','TUBE',ICOIL,PAR,3,IVOLU)
      PAR(1) = RSOIN
      PAR(2) = RSOOUT
      PAR(3) = LSOBW/2.0
      CALL GSVOLU('SOBW','TUBE',ICOIL,PAR,3,IVOLU)

      CALL GSPOS('VESL',1,'MAST',0,0,0,0,'ONLY')
      CALL GSPOS('SOMA',1,'VESL',0,0,0,0,'ONLY')
      CALL GSPOS('SOFW',1,'SOMA',0,0,ZSOFW,0,'0NLY')
      CALL GSPOS('SOMD',1,'SOMA',0,0,ZSOMD,0,'0NLY')
      CALL GSPOS('SOBW',1,'SOMA',0,0,ZSOBW,0,'0NLY')

*-- =====  TEMPLATE FOR RETURN YOKE GEOMETRY  =====

*-- DEFINE GEOMETRY
      PAR(1) = FIMIN
      PAR(2) = FISPAN
      PAR(3) = NSIDE
      PAR(4) = NZ
      PAR(5) = ZB
      PAR(6) = RMIN
      PAR(7) = RMAX
      PAR(8) = ZF
      PAR(9) = RMIN
      PAR(10)= RMAX
      CALL GSVOLU('FEMA','PGON',IAIRJM,PAR,10,IVOLU)
      CALL GSPOS('FEMA',1,'MAST',0,0,0,0,'ONLY')

      DO 1   IL=1,5
         WRITE(CDATA1(4),'(I1)') IL
         CNAME  = CDATA1(1) // CDATA1(2) // CDATA1(3) // CDATA1(4)
         PAR(1) = FIMIN
         PAR(2) = FISPAN
         PAR(3) = NSIDE
         PAR(4) = NZ
         PAR(5) = ZB
         PAR(6) = RFEIN(IL)
         PAR(7) = RFEOUT(IL)
         PAR(8) = ZF
         PAR(9) = RFEIN(IL)
         PAR(10)= RFEOUT(IL)
         CALL GSVOLU(CNAME,'PGON',IIRONB,PAR,10,IVOLU)
         CALL GSPOS(CNAME,1,'FEMA',0,0,0,0,'0NLY')
 1    CONTINUE

*-- =====  TEMPLATE FOR ENDCAP RETURN YOKE GEOMETRY  =====
*--             94-JUNE-15, Y.FUJII

*-- ALL VALUES ARE GIVEN IN CGS ; cm, g/cm3, and degree.

*-- DEFINE FRONT SECTION GEOMETRY ( FOR BOTH FORWARD AND BACKWARD )

      PAR(1) = RECIN(1)
      PAR(2) = RECOUT(1)
      PAR(3) = (ZEC2(1)-ZEC1(1))/2.0
      ZECF1  = (ZEC2(1)+ZEC1(1))/2.0
      CALL GSVOLU('ECF1','TUBE',IIRONE,PAR,3,IVOLU)
      CALL GSPOS('ECF1',1,'MAST',0,0,ZECF1,0,'ONLY')

      PAR(1) = RECIN(1)
      PAR(2) = RECOUT(1)
      PAR(3) = (ZEC2(1)-ZEC1(1))/2.0
      ZECF1  =-(ZEC2(1)+ZEC1(1))/2.0
      CALL GSVOLU('ECB1','TUBE',IIRONE,PAR,3,IVOLU)
      CALL GSPOS('ECB1',1,'MAST',0,0,ZECF1,0,'ONLY')

*-- DEFINE REAR SECTION GEOMETRY ( FIVE LAYERS )
      DO 2   IFB=1,2
         IF ( IFB .EQ. 1 )   THEN
             CDATA2(3) = 'F'
         ELSE
             CDATA2(3) = 'B'
         ENDIF
         DO 3   IL=2,6
            WRITE(CDATA2(4),'(I1)') IL
            CNAME  = CDATA2(1) // CDATA2(2) // CDATA2(3) // CDATA2(4)
            PAR(1) = FIMIN
            PAR(2) = FISPAN
            PAR(3) = NSIDE
            PAR(4) = NZ
            IF ( IFB .EQ. 1 )   THEN
              PAR(5) = ZEC1(IL) 
              PAR(8) = ZEC2(IL) 
            ELSE
              PAR(5) = -ZEC2(IL) 
              PAR(8) = -ZEC1(IL) 
            ENDIF
            PAR(6) = RECIN(2)
            PAR(7) = RECOUT(2)
            PAR(9) = RECIN(2)
            PAR(10)= RECOUT(2)
            CALL GSVOLU(CNAME,'PGON',IIRONE,PAR,10,IVOLU)
            CALL GSPOS(CNAME,1,'MAST',0,0,0,0,'0NLY')
 3       CONTINUE
 2    CONTINUE

      RETURN
      END



