      SUBROUTINE VXDIGI
*.
*...VXDIGI   Compute and store DIGIts for Si vertex detector
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : VXGEO  GCNUM
*. CALLS     : GFHITS KZBCR1 KZPUT1 JGP2LP
*. CALLED    : GUDIGI
*.
*. BANKS L   : MCVXHIT
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Naoyasu HOSODA
*. VERSION   : 1.00
*. CREATED   : 14-Apr-1994
*. LAST MOD  : 29-Jul-1994
*.
*. MODIFICATION LOG
*. 08-Jul-1994 N.HOSODA, Add MCVXHIT bank
*. 29-Jul-1994 N.HOSODA, Use JGP2LP
*. 09-May-1995 Y.Fujii,  ID limited to 3. Add comments.
*.
*. Bank Data Structure ;
*. name = MCVXHIT
*.        First Two Words #1  total number of hits
*.                        #2  total number of tracks
*.        Ten Words/Hit   #1  track number responsible to this hit
*.                        #2  layer number of this hit
*.                        #3  pixel number in theta direction
*.                        #4  pixel number in phai direction
*.                        #5  particle code
*.                        #6  x coordinate of this hit [cm]
*.                        #7  y coordinate of this hit [cm]
*.                        #8  z coordinate of this hit [cm]
*.                        #9  p of particle generated this hit [GeV]
*.                        #10 energy deposit to this pixel [keV]
*.
*.--  21-Apr-1999:
*.--  The following elements (11-20) are filled if
*.--             BSIZ 2
*.--  is given in CT data cards
*.
*.                        #11 Px of particle [GeV]
*.                        #12 Py of particle [GeV]
*.                        #13 Pz of particle [GeV]
*.                        #14  particle number of previous track
*.                        #15  vertex position in x
*.                        #16  vertex position in y
*.                        #17  vertex position in z
*.                        #18  momentum-x at vertex position
*.                        #19  momentum-y at vertex position
*.                        #20  momentum-z at vertex position
*.
*.**********************************************************************
      IMPLICIT NONE

*--   Common
#include "geant321/gcnum.inc"

#include "vxgeo.inc"
#include "vxsim.inc"
#include "vxhit.inc"

      INTEGER NUMVS(NDSVVX),NUMBV(NDSVVX,NHMXVX),ITRA(NHMXVX)
      REAL    HITS(NPHTVX,NHMXVX)

      INTEGER NDATMX
      PARAMETER (NDATMX=20)
      INTEGER   IDATA(NDATMX)
      REAL      RDATA(NDATMX)
      EQUIVALENCE (IDATA(1), RDATA(1))
      CHARACTER*32 FMSVXH,FSGVXH,FSSVXH,FLSVXH
c      CHARACTER*32 CNAVXH,FMSVXH,FSSVXH
      INTEGER NWMVXH,LSGVXH,LSSVXH,LLSVXH,NWPVXH,NDAVXH
*--   Parameter for bank
c      PARAMETER (CNAVXH='MCVXHIT')
      PARAMETER (NWMVXH=2)
      PARAMETER (FMSVXH='2I')
      PARAMETER (LSSVXH=10, LLSVXH=20)
      PARAMETER (FSSVXH='5I 5F')
      PARAMETER (FLSVXH='5I 8F 1I 6F')
      PARAMETER (NWPVXH=0)
      PARAMETER (NDAVXH=10)

      INTEGER NUMHIT,IHIT,NHITS
      INTEGER IHT,ILPC
      INTEGER IERVXH

      CHARACTER*80 CMSG
      INTEGER LNBLNK

      INTEGER I,J,IERR

*--   Initialize
      DO 10 I=1,NDSVVX
        NUMVS(I)=0
 10   CONTINUE

      NUMHIT=0
      DO 20 I=1,NLAYVX
         CALL GFHITS('VX  ',VNAMVX(I),NDSVVX,NPHTVX,NHMXVX,0,NUMVS,
     &        ITRA,NUMBV,HITS,NHITS)
         IF (NHITS.GT.NHMXVX) THEN
            WRITE(CMSG,'(''Nhits overflow for layer '',I1)') I
            CALL MSGOUT('VXDIGI',CMSG(1:LNBLNK(CMSG)),NHMXVX)
            NHITS = NHITS-1
         ENDIF    
         NUMHIT = NUMHIT+NHITS
 20   CONTINUE

      IF (IBNKVX.EQ.ISBKVX) THEN
         LSGVXH = LSSVXH
         FSGVXH = FSSVXH
      ELSEIF (IBNKVX.EQ.ILBKVX) THEN
         LSGVXH = LLSVXH
         FSGVXH = FLSVXH
      ELSE
         CALL MSGOUT('VXDIGI','Wrong bank type',IBNKVX)
         STOP
      ENDIF

*-- Create new ZBS bank with the 1-dim. fixed length segment.
c      print *,'vxdigi;numhit = ',numhit
      CALL KZBCR1('MCVXHIT',NWMVXH,FMSVXH,NUMHIT,LSGVXH,FSGVXH, IERR)
      
      IHIT = 0
      DO 100 I=1,NLAYVX
         CALL GFHITS('VX  ',VNAMVX(I),NDSVVX,NPHTVX,NHMXVX,0,NUMVS,
     &        ITRA,NUMBV,HITS,NHITS)
         IF (NHITS.GT.NHMXVX) THEN
            NHITS = NHMXVX
         ENDIF    
         
*-- Loop of hit in this detector name
         DO 110 IHT=1,NHITS
            IHIT = IHIT+1
            IDATA(1) = ITRA(IHT)
            IDATA(2) = NUMBV(1,IHT)
            IDATA(3) = NUMBV(2,IHT)
            IDATA(4) = NUMBV(3,IHT)
            
*-- Convert geant p-code to LUND74 P-code
            CALL JGP2LP(NINT(HITS(1,IHT)), ILPC,IERR)
            IF (IERR .EQ. 0) THEN
               IDATA(5) =ILPC
            ELSE
               IDATA(5) =9999
            ENDIF
            RDATA(6) = HITS(2,IHT)
            RDATA(7) = HITS(3,IHT)
            RDATA(8) = HITS(4,IHT)
            RDATA(9) = HITS(5,IHT)

            IF (IBNKVX.EQ.ISBKVX) THEN

               RDATA(10)= HITS(6,IHT)

            ELSEIF (IBNKVX.EQ.ILBKVX) THEN

               RDATA(10)= HITS(16,IHT)

               CALL JGP2LP(NINT(HITS(9,IHT)), ILPC,IERR)
               IF (IERR .EQ. 0) THEN
                  IDATA(14) =ILPC
               ELSE
                  IDATA(14) =9999
               ENDIF
               RDATA(11)= HITS(6,IHT)
               RDATA(12)= HITS(7,IHT)
               RDATA(13)= HITS(8,IHT)
               RDATA(15)= HITS(10,IHT)
               RDATA(16)= HITS(11,IHT)
               RDATA(17)= HITS(12,IHT)
               RDATA(18)= HITS(13,IHT)
               RDATA(19)= HITS(14,IHT)
               RDATA(20)= HITS(15,IHT)

            ENDIF

*-- Put data words into the given segment of the bank
c            print *,'vxdigi;ihit = ',ihit
c            print *,'       ;idata = ',(idata(j),j=1,5)
c            print *,'       ;rdata = ',(rdata(j),j=6,10)
            CALL KZREP1('MCVXHIT',IHIT,LSGVXH,IDATA)

 110     CONTINUE

  100 CONTINUE


*-- Put data words into the master segment of the bank
      IDATA(1) = NUMHIT
      IDATA(2) = NTRACK
      CALL KZPUT1('MCVXHIT',0,0,NWMVXH,IDATA)

      RETURN
      END
