      SUBROUTINE FTCIRC(NCIR, XCIR, YCIR, SIGCIR,
     &   SIG, RC, PHI0, RMIN, ERRMAT, CIRPUL, CHISQ )

*.
*...FTCIRC   subroutine to fit circle.
*.
*.  INPUT    : NCIR    number of hits
*.           : XCIR    x value of hit points
*.           : YCIR    y value of hit points
*.           : SIGCIR  standard deviation of x-y values
*.  OUTPUT   : SIG     sign of angle
*.           : PHI0    phi value of center of circle
*.           :         if (SIG =  1) then
*.           :           phi = PHI0 - PI/2
*.           :         if (SIG = -1) then
*.           :           phi = PHI0 + PI/2
*.           : RC      radius of circle.
*.           : RMIN    closest approach to the origin.
*.           :         X = ( RC - RMIN )*COS(phi)
*.           :         Y = ( RC - RMIN )*SIN(phi)
*.           : ERRM    error matrix of the parameters
*.           :   ERRM(i,j). i and j are the parameter
*.           :         1--1/Rc, 2--Phi0 and 3--Rmin
*.           :   The error matrix is the invert matrix of
*.           :   ERRM(1,1) =           = d2(chisq)/d(1/Rc)d(1/Rc)
*.           :   ERRM(1,2) = ERRM(2,1) = d2(chisq)/d(1/Rc)d(Phi0)
*.           :   ERRM(1,3) = ERRM(3,1) = d2(chisq)/d(1/Rc)d(Rmin)
*.           :   ERRM(2,2)             = d2(chisq)/d(Phi0)d(Phi0)
*.           :   ERRM(2,3) = ERRM(3,2) = d2(chisq)/d(Phi0)d(Rmin)
*.           :   ERRM(3,3) =           = d2(chisq)/d(Rmin)d(Rmin)
*.           : CIRPUL  residuals for the given data
*.           : CHISQ   chi squared
*.
*. AUTHOR    : J. Kanzaki
*. VERSION   : 1.00
*. CREATED   : 06-Mar-1996
*. LAST MOD  : 06-Mar-1996
*.
*. HISTORY   :
*. DD-MMM-YY  CREATED BY  H.Boerner
*. 31-Jul-1985  CHANGED BY  Yuji Nakagawa (TMU)
*. 04-Apr-1987  Modified to be fast by T.Tsuboyama(KEK)
*. 14-Aug-1987  Error matrix is available.
*.
      IMPLICIT NONE
      INTEGER NCIR
      REAL XCIR(NCIR),YCIR(NCIR),SIGCIR(NCIR),CIRPUL(NCIR)
C     REAL SIG, RC, PHI0, RMIN, ERRM(3,3), CHISQ
      REAL SIG, RC, PHI0, RMIN, CHISQ

      DOUBLE PRECISION KAPPA, LAMDA
C Covariant matrix for (RC,RMIN,PHI0),(Intermediate),(X0,Y0,RC) and
C (transformation matrix from RC,RMIN,PHI0 to X0,Y0,C0).
      DOUBLE PRECISION ERRMAT(3,3)
C====================================================================C
      REAL PI,TWOPI
      PARAMETER (PI=3.14159265358979323846, TWOPI=PI*2)
      INTEGER NTRY,ITRY
      PARAMETER (NTRY=5)

      DOUBLE PRECISION WEIGHT,DXCIR,DYCIR
      DOUBLE PRECISION WSUM,XAV,YAV,XXAV,XYAV,YYAV
      DOUBLE PRECISION XXXAV,XXYAV,XYYAV,YYYAV,RRRRAV

      DOUBLE PRECISION A,B,ASQPB,RASQPB,SPLUS,SMINUS
      DOUBLE PRECISION COSROT,SINROT
      DOUBLE PRECISION RRAV,RSCALE,RSC2,RSC3

      DOUBLE PRECISION XXAV1,XRRAV,YRRAV
      DOUBLE PRECISION H11,H14,H22,H24,H34
      DOUBLE PRECISION XRRXRR,YRRYRR,RRRRM1,XXYY
      DOUBLE PRECISION C0,C1,C2,C2D
      DOUBLE PRECISION CHISCL,DLAMDA,DLAMAX,ROOTSQ
      DOUBLE PRECISION RATIO,ALPHA,BETA,GAMMA
      DOUBLE PRECISION ALPHAR,BETAR
      DOUBLE PRECISION DRMIN,RINV,BUNBO
      DOUBLE PRECISION SINPHI,COSPHI,CCHISQ

      DOUBLE PRECISION X,Y
      DOUBLE PRECISION DIR0,DIR1,DELTA,DELK,DELP,DELR
      DOUBLE PRECISION DELKK,DELPP,DELRR,DELKP,DELKR,DELPR

      INTEGER I,J

C====================================================================C
      IF (NCIR.LT.3) RETURN
C===== Calculates averages used in the h matrix of the circle fit
C===== H is calculated in the coordinate system where :
C===== <x>=<y>=<xy>=0 and <r**2> = 1
C===== in this coordinate system, only 5 elements of h are non zero.
C===== they are: <x**2>, <y**2>, <x r**2>, <y r**2>, <r**4>.
C===== <x> and <y> before the translation and COSROT & SINROT
C===== are needed to get back to the lab system.
C===== SIGN(q) can be determined by the signs chosen for COSROT
C===== and SINROT wsum is required to scale the chisquare
C===== rscale is also required to scale gamma & kappa
C===== First, calculate <x> and <y> in the original coordinate system
C
C
      WSUM   = 0D0
      XAV    = 0D0
      YAV    = 0D0
      XXAV   = 0D0
      XYAV   = 0D0
      YYAV   = 0D0
      XXXAV  = 0D0
      XXYAV  = 0D0
      XYYAV  = 0D0
      YYYAV  = 0D0
      RRRRAV = 0D0
C
      DO 1 I=1,NCIR
          WEIGHT=1.0D0/DBLE(SIGCIR(I))**2
          DXCIR=XCIR(I)
          DYCIR=YCIR(I)
          WSUM  = WSUM  + WEIGHT
          XAV   = XAV   + WEIGHT * DXCIR
          XXAV  = XXAV  + WEIGHT * DXCIR**2
          XXXAV = XXXAV + WEIGHT * DXCIR**3
          YAV   = YAV   + WEIGHT             *DYCIR
          YYAV  = YYAV  + WEIGHT             *DYCIR**2
          YYYAV = YYYAV + WEIGHT             *DYCIR**3
          XYAV  = XYAV  + WEIGHT * DXCIR     *DYCIR
          XXYAV = XXYAV + WEIGHT * DXCIR**2  *DYCIR
          XYYAV = XYYAV + WEIGHT * DXCIR     *DYCIR**2
          RRRRAV= RRRRAV+ WEIGHT *(DXCIR**2  +DYCIR**2)**2
1     CONTINUE
C
      XAV   = XAV   /WSUM
      YAV   = YAV   /WSUM
      XXAV  = XXAV  /WSUM
      XYAV  = XYAV  /WSUM
      YYAV  = YYAV  /WSUM
      XXXAV = XXXAV /WSUM
      XXYAV = XXYAV /WSUM
      XYYAV = XYYAV /WSUM
      YYYAV = YYYAV /WSUM
      RRRRAV= RRRRAV/WSUM
C
C===== Translate into the frame where  <x>=0, <y>=0
C
      RRRRAV = RRRRAV
     >         -4*((XXXAV+XYYAV)*XAV     +(YYYAV+XXYAV)*YAV )
     >         +XXAV*(6*XAV**2+2*YAV**2) +YYAV*(6*YAV**2+2*XAV**2 )
     >         +8*XYAV*XAV*YAV           -3*(XAV**2 +YAV**2 )**2
      XXXAV = XXXAV -3*XXAV*XAV +2*XAV**3
      XXYAV = XXYAV -2*XAV*XYAV +YAV*(2*XAV**2 - XXAV)
      XYYAV = XYYAV -2*YAV*XYAV +XAV*(2*YAV**2 - YYAV)
      YYYAV = YYYAV -3*YYAV*YAV +2*YAV**3
      XXAV  = XXAV  -XAV**2
      XYAV  = XYAV  -XAV*YAV
      YYAV  = YYAV  -YAV**2
C
C===== Determin the rotation angle where,
C===== <xy> = 0 and sign(c**2 - s**2) = sign(xxav-yyav) and
C===== (xxav-yyav) > 0 and & sign(s) = sign(xyav)

      A = DABS(XXAV - YYAV)
      B = 4*XYAV*XYAV

      ASQPB   = A**2 + B
      RASQPB  = SQRT(ASQPB)

      SPLUS  = 1D0 + A / RASQPB
      SMINUS = B/(ASQPB * SPLUS)

      SPLUS  = SQRT( SPLUS/2)
      SMINUS = SQRT(SMINUS/2)

C===== First require: SIGN(c**2 - s**2) = sign(xxav - yyav)

      IF (XXAV .LE. YYAV) THEN
        COSROT = SMINUS
        SINROT = SPLUS
       ELSE
        COSROT = SPLUS
        SINROT = SMINUS
      ENDIF

C===== Require: SIGN(s) = SIGN(xyav) * SIGN(c) (assuming SIGN(c) > 0)

      IF (XYAV .LT. 0D0) SINROT = - SINROT

C===== We now have the smallest angle that guarantees <x**2> > <y**2>.
C===== To get the sign of the charge right, the new x-axis must point
C===== outward from the origin. We are free to change signs of both
C===== COSROT and SINROT simultaneously to accomplish this.
C
C===== Choose the signs wisely to be able to get the sign of the charge.
      IF ((COSROT*XAV + SINROT*YAV) .LE. 0D0) THEN
        COSROT = -COSROT
        SINROT = -SINROT
      ENDIF

C===== Now get the scale factor for  <r**2> = 1

      RRAV  = XXAV + YYAV
      RSCALE = SQRT(RRAV)
      RSC2 = 1/RRAV
      RSC3 = RSC2/RSCALE

C===== rotate and scale.

      RRRRAV = RRRRAV/RRAV**2
      XXAV1=(XXAV*COSROT**2+2*XYAV*COSROT*SINROT+YYAV*SINROT**2 )*RSC2
      YYAV =(YYAV*COSROT**2-2*XYAV*COSROT*SINROT+XXAV*SINROT**2 )*RSC2
      XXAV = XXAV1
      XRRAV= ((XXXAV+XYYAV)*COSROT + (YYYAV+XXYAV)*SINROT )*RSC3
      YRRAV= ((YYYAV+XXYAV)*COSROT - (XXXAV+XYYAV)*SINROT )*RSC3
      IF (NCIR.EQ.3) THEN
C In this case, only the exact solution is available.
        CHISQ =0.0
        H11   = XXAV
        H22   = YYAV
        H34   = 1D0
       ELSE
C Use thest to get the coefficients of the 4-th order polynomial.
        XRRXRR = XRRAV **2
        YRRYRR = YRRAV **2
        RRRRM1 = RRRRAV  -1D0
        XXYY   = XXAV * YYAV
        C0 =          RRRRM1*XXYY - XRRXRR*YYAV - YRRYRR*XXAV
        C1 =        - RRRRM1      + XRRXRR      + YRRYRR      - 4 *XXYY
C       C3 = 0.0
        C2 =   4D0  + RRRRM1                                  - 4 *XXYY
C       C4 = -4.0
C=====coefficients of the derivative - used in NEWTON-RAPHSON iterations
C       C1D = 1 * C1
        C2D = 2 * C2
C       C3D = 3 * C3
C       C4D = 4 * C4
        LAMDA  = 0D0
        CHISCL = WSUM * RSCALE**2
        DLAMAX = 0.001D0 / CHISCL
        DO 2 ITRY=1,NTRY
          DLAMDA = -( -4 *LAMDA**4 +C2 *LAMDA**2 +C1*LAMDA +C0 )/
     >              ( -16*LAMDA**3 +C2D*LAMDA    +C1)
          LAMDA  = LAMDA + DLAMDA
          IF ( DABS( DLAMDA ) .LT. DLAMAX ) GOTO 3
2       CONTINUE
3       CONTINUE
        CHISQ  = CHISCL  *LAMDA
C===== now calculate the matrix elements for ALPHA, BETA, GAMMA & KAPPA
        H11    = XXAV -   LAMDA
        H22    = YYAV -   LAMDA
        H34    = 1D0  + 2*LAMDA
      ENDIF
      H24  = YRRAV
      H14  = XRRAV
      ROOTSQ = ( H14/H11 )**2 + 4 *H34
      IF ( DABS( H22 ) .GE. DABS( H24 )) THEN
        RATIO  = H24/H22
        ROOTSQ = RATIO**2 + ROOTSQ
        KAPPA  = 1D0 / SQRT( ROOTSQ )
        BETA   = -RATIO * KAPPA
       ELSE
        RATIO  = H22/H24
        ROOTSQ = 1D0 + ROOTSQ * RATIO**2
        BETA   = 1D0 / SQRT( ROOTSQ )
        IF ( H24 .GT. 0.0 )  BETA = -BETA
        KAPPA  = - RATIO*BETA
      ENDIF
      ALPHA = -( H14/H11 ) * KAPPA
      GAMMA = -  H34       * KAPPA
C===== transform them into the lab coordinate system.
C===== first get KAPPA and GAMMA back to real dimensions.
      KAPPA = KAPPA / RSCALE
      GAMMA = GAMMA * RSCALE
C===== next rotate ALPHA and BETA.
      ALPHAR= COSROT*ALPHA - SINROT*BETA
      BETAR = SINROT*ALPHA + COSROT*BETA
C===== then translate by (XAV,YAV).
      ALPHA = ALPHAR - 2 *KAPPA *XAV
      BETA  = BETAR  - 2 *KAPPA *YAV
      GAMMA = KAPPA * (XAV**2 + YAV**2) - ALPHAR*XAV - BETAR*YAV + GAMMA
C We now have ALPHA,BETA,GAMMA and KAPPA in the real space.
C Results
      SIG   = -DSIGN ( 1.0D+00,YRRAV)
      RC = 0.5/KAPPA
      RMIN = -2*GAMMA/(1.0 +SQRT(1.0 +4*KAPPA*GAMMA))
      PHI0 = -ATAN2( ALPHA, BETA )
      IF (PHI0 .LT. 0.0) PHI0 = PHI0 + TWOPI
      IF (SIG  .LT. 0.0) PHI0 = PHI0 -    PI

      DO 4  I=1,NCIR
         CIRPUL(I) =(KAPPA*(XCIR(I)**2  +YCIR(I)**2)
     >        +ALPHA*XCIR(I)+BETA*YCIR(I)+GAMMA)/SIGCIR(I)
 4    CONTINUE

      DRMIN = -2*GAMMA/(1.0 +SQRT(1.0 +4*KAPPA*GAMMA))
      RINV  =  2*KAPPA
      BUNBO = 1.0D0-RINV*DRMIN
      IF (BUNBO.LT.0.00001) THEN
         CALL MSGOUT('FTCIRC','BUNBO = 0',1)
c        WRITE(6,*) 'CIRFER:BUNBO=0'
         SINPHI=SIN(PHI0)
         COSPHI=COS(PHI0)
      ELSE
         SINPHI=-ALPHA/BUNBO
         COSPHI= BETA /BUNBO
      ENDIF
      ERRMAT(1,1) = 0.0
      ERRMAT(1,2) = 0.0
      ERRMAT(1,3) = 0.0
      ERRMAT(2,2) = 0.0
      ERRMAT(2,3) = 0.0
      ERRMAT(3,3) = 0.0
      CCHISQ=0.0
      DO 5 I = 1,NCIR
          WEIGHT=1.0D0/DBLE(SIGCIR(I))**2
          X=XCIR(I)
          Y=YCIR(I)
          DIR0=-X*SINPHI+Y*COSPHI
          DIR1=-X*COSPHI-Y*SINPHI
          DELTA=RINV*(X**2+Y**2+DRMIN**2-2.*DRMIN*DIR0)/2+(DIR0-DRMIN)
          DELK =(X**2+Y**2+DRMIN**2)/2.0 -DRMIN*DIR0
          DELP =-DIR1*(RINV*DRMIN-1.)
          DELR =RINV*(DRMIN-DIR0)-1.
          DELKK=0.
          DELPP=DIR0*(RINV*DRMIN-1.)
          DELRR=RINV
          DELKP=-DRMIN*DIR1
          DELKR= DRMIN-DIR0
          DELPR=-RINV*DIR1
          CCHISQ      = CCHISQ      + DELTA*DELTA             *WEIGHT
C 1=1/RC  2=PHI0  3=RMIN
          ERRMAT(1,1) = ERRMAT(1,1) +(DELK*DELK + DELTA*DELKK)*WEIGHT
          ERRMAT(1,2) = ERRMAT(1,2) +(DELK*DELP + DELTA*DELKP)*WEIGHT
          ERRMAT(1,3) = ERRMAT(1,3) +(DELK*DELR + DELTA*DELKR)*WEIGHT
          ERRMAT(2,2) = ERRMAT(2,2) +(DELP*DELP + DELTA*DELPP)*WEIGHT
          ERRMAT(2,3) = ERRMAT(2,3) +(DELP*DELR + DELTA*DELPR)*WEIGHT
          ERRMAT(3,3) = ERRMAT(3,3) +(DELR*DELR + DELTA*DELRR)*WEIGHT
5     CONTINUE
      ERRMAT(3,1) = ERRMAT(1,3)
      ERRMAT(3,2) = ERRMAT(2,3)
      ERRMAT(2,1) = ERRMAT(1,2)
C  Get the inverse matrix. The result is the error matrix.
      CALL DMTINV(ERRMAT,3)
      IF (DABS(DBLE(CHISQ)-CCHISQ).GT.CHISQ*0.01) THEN
         CALL MSGOUT('FTCIRC','CHISQ is inconsistent',2)
c        WRITE(6,*) 'CIRFER:CHISQ is inconsistent.',CHISQ,CCHISQ
      ENDIF
C     WRITE(6,'(A,2F20.10)') ' RC  =',  RC,SQRT(ERRMAT(1,1))*RC**2
C     WRITE(6,'(A,2F20.10)') ' PHI0=',PHI0,SQRT(ERRMAT(2,2))
C     WRITE(6,'(A,2F20.10)') ' RMIN=',RMIN,SQRT(ERRMAT(3,3))
C      DO 110 I=1,3
C      DO 120 J=1,3
C      ERRM(J,I)=ERRMAT(J,I)
C 120  CONTINUE
C 110  CONTINUE
      RETURN
      END
