      SUBROUTINE GETCAL(IERR)
*.
*...GETCAL   obtain calorimeter hits information 
*.           and store to TRBUFF, CALDAT
*.           
*. INPUT     : IFS =1 full JIM, =2 smear JIM
*. OUTPUT    : IERR =0 no error, =1 at least one BANK does not exist
*.                  =2 NHIT exceed max hits(3000)
*.                  =3 NEMH exceed max hits(500)
*.                  =4 NHDH exceed max hits(400)
*.           
*.           
*. REPORT CONDITIONS
*.
*.  TEMPORARY: TRBUFF bank is filled with the follwing condition.  
*.    EMCAL  >>> Cell energy is obtained by EM(electron/positron/gamma)
*.               calibration factor. Theta dependence is included.
*.    HADCAL >>> Cell energy is obtained by hadron calibration factor.
*.               Theta dependence is included.
*.  e/pi ratio of JIM(FULL/SMEAR) calorimeter is about 1.1.
*.  To calibrate correctly we need a particle ID for each cluster.
*.  In above condition a visible energy of hadron track is expected 
*.  to be small by a several %. 
*.  For example a visible energy of 10 GeV pion was found to be 9.36 GeV. 
*.  Energy resolution is also worse.
*.                                         06.Mar.1999   Koji ISHII
*.
*.
*. AUTHOR    : Koji ISHII
*. VERSION   : 1.00
*. CREATED   : 06-Mar-1999
*. LAST MOD  : 06-Mar-1999
*.
*. MODIFICATION LOG
*. 20-Apr-1999, Addressing scheme is changed for End-cap part to fit
*.              SIMDST convention, J.Kanzaki
*. 20-Apr-1999, Low energy hits in smearing mode (<1MeV) is discarded,
*.              J.Kanzaki
*.
*.**********************************************************************

      IMPLICIT NONE

#include "buffer.inc"
#include "trbuff.inc"
#include "jmdet.inc"
#include "caldat.inc"
#include "calpos.inc"

      INTEGER LBANK,IERR,I
      
      INTEGER IHIT,NHIT,NEHIT,NHHIT
      INTEGER ITHET,MFLAG,ISTA,IEND
      
      IERR = 0
      
      NHIT = 0
      NEHIT = 0
      NHHIT = 0
      
      CALL VZERO(RCALH,3*MXRCAL)
      CALL VZERO(NCALH,9)
      CALL VZERO(EMH,NPECLS*MXECLS)
      CALL VZERO(HDH,NPHCLS*MXHCLS)
      
      CALL KZBLOC('MCBEHIT',LBANK)
      IF (LBANK.LE.0) THEN
         CALL MSGOUT('GETCAL','MCBEHIT does not exist',LBANK)
         IERR = 1
      ELSE
         CALL KZGET('MCBEHIT',0,NBUFF,IBUFF)
         NCALH(1) = IBUFF(1)
         
         DO 100 IHIT =1, NCALH(1)
            NHIT = NHIT +1
            CALL KZGET('MCBEHIT',IHIT,NBUFF,IBUFF)
            RCALH(1,NHIT) = RBUFF(2)
            RCALH(2,NHIT) = IBUFF(1)
            RCALH(3,NHIT) = 111.
            ITHET = IBUFF(1) / 1000
            NEHIT = NEHIT + 1
            EMH(1,NEHIT) = 1000*RBUFF(2)*BEFACT(3,ITHET)
            EMH(2,NEHIT) = IBUFF(1)
 100     CONTINUE
      ENDIF 
         
      CALL KZBLOC('MCBHHIT',LBANK)
      IF (LBANK.LE.0) THEN
         CALL MSGOUT('GETCAL','MCBHHIT does not exist',LBANK)
         IERR = 1
      ELSE
         CALL KZGET('MCBHHIT',0,NBUFF,IBUFF)
         NCALH(2) = IBUFF(1)
         
         DO 150 IHIT =1, NCALH(2)
            NHIT = NHIT +1
            CALL KZGET('MCBHHIT',IHIT,NBUFF,IBUFF)
            RCALH(1,NHIT) = RBUFF(2)+RBUFF(3)
            RCALH(2,NHIT) = IBUFF(1)
            RCALH(3,NHIT) = 112.
            ITHET = IBUFF(1) / 1000
            NHHIT = NHHIT + 1
            HDH(1,NHHIT) = 1000*(RBUFF(2)+RBUFF(3))*BHFACT(4,ITHET)
            HDH(2,NHHIT) = IBUFF(1)
 150     CONTINUE
      ENDIF 
         
      CALL KZBLOC('MCEEHIT',LBANK)
      IF (LBANK.LE.0) THEN
         CALL MSGOUT('GETCAL','MCEEHIT does not exist',LBANK)
         IERR = 1
      ELSE 
         CALL KZGET('MCEEHIT',0,NBUFF,IBUFF)
         NCALH(3) = IBUFF(1)
         
         DO 200 IHIT =1, NCALH(3)
            NHIT = NHIT +1
            CALL KZGET('MCEEHIT',IHIT,NBUFF,IBUFF)
            IBUFF(1) = IBUFF(1)+ISIGN(1000000,IBUFF(1))
            RCALH(1,NHIT) = RBUFF(2)
            RCALH(2,NHIT) = IBUFF(1)
            RCALH(3,NHIT) = 121.
            ITHET = ABS(MOD(IBUFF(1),1000000)) / 1000
            NEHIT = NEHIT + 1
            EMH(1,NEHIT) = 1000*RBUFF(2)*EEFACT(3,ITHET)
            EMH(2,NEHIT) = IBUFF(1)
 200     CONTINUE
      ENDIF
      
      CALL KZBLOC('MCEHHIT',LBANK)
      IF (LBANK.LE.0) THEN
         CALL MSGOUT('GETCAL','MCEHHIT does not exist',LBANK)
         IERR = 1
      ELSE 
         CALL KZGET('MCEHHIT',0,NBUFF,IBUFF)
         NCALH(4) = IBUFF(1)
         
         DO 250 IHIT =1, NCALH(4)
            NHIT = NHIT +1
            CALL KZGET('MCEHHIT',IHIT,NBUFF,IBUFF)
            IBUFF(1) = IBUFF(1)+ISIGN(1000000,IBUFF(1))
            RCALH(1,NHIT) = RBUFF(2)+RBUFF(3)
            RCALH(2,NHIT) = IBUFF(1)
            RCALH(3,NHIT) = 122.
            ITHET = ABS(MOD(IBUFF(1),1000000)) / 1000
            NHHIT = NHHIT + 1
            HDH(1,NHHIT) = 1000*(RBUFF(2)+RBUFF(3))*EHFACT(4,ITHET)
            HDH(2,NHHIT) = IBUFF(1)
 250     CONTINUE
      ENDIF
      
* If full JIM, skip smear BANK

      IF (ISIMJD(IDETBC).EQ.ISMRJD) THEN

         CALL KZBLOC('MCBESMH',LBANK)
         IF (LBANK.LE.0) THEN
            CALL MSGOUT('GETCAL','MCBESMH does not exist',LBANK)
            IERR = 1
         ELSE
            CALL KZGET('MCBESMH',0,NBUFF,IBUFF)
            NCALH(5) = IBUFF(1)
            ISTA = 1
            IEND = NCALH(1)
            
            DO 300 IHIT =1, NCALH(5)
               NHIT = NHIT +1
               CALL KZGET('MCBESMH',IHIT,NBUFF,IBUFF)
               RCALH(1,NHIT) = RBUFF(2)
               RCALH(2,NHIT) = IBUFF(1)
               RCALH(3,NHIT) = 211.
               ITHET = IBUFF(1) / 1000
               MFLAG=0
               DO 310 I=ISTA,IEND
                  IF(IBUFF(1).EQ.INT(EMH(2,I))) THEN
                     MFLAG=1
                     EMH(1,I)=EMH(1,I)+1000*RBUFF(2)*BEFACT(1,ITHET)
                     GOTO 310
                  ENDIF
 310           CONTINUE
               IF(MFLAG.EQ.0) THEN
                  IF (INT(1000.*RBUFF(2)*BEFACT(1,ITHET)).GT.0) THEN
                     NEHIT = NEHIT + 1
                     EMH(1,NEHIT) = 1000*RBUFF(2)*BEFACT(1,ITHET)
                     EMH(2,NEHIT) = IBUFF(1)
                  ENDIF
               ENDIF
 300        CONTINUE
         ENDIF 
         
         CALL KZBLOC('MCBHSMH',LBANK)
         IF (LBANK.LE.0) THEN
            CALL MSGOUT('GETCAL','MCBHSMH does not exist',LBANK)
            IERR = 1
         ELSE
            CALL KZGET('MCBHSMH',0,NBUFF,IBUFF)
            NCALH(6) = IBUFF(1)
            ISTA = 1
            IEND = NCALH(2)
            
            DO 350 IHIT =1, NCALH(6)
               NHIT = NHIT +1
               CALL KZGET('MCBHSMH',IHIT,NBUFF,IBUFF)
               RCALH(1,NHIT) = RBUFF(2)
               RCALH(2,NHIT) = IBUFF(1)
               RCALH(3,NHIT) = 212.
               ITHET = IBUFF(1) / 1000
               MFLAG=0
               DO 360 I=ISTA,IEND
                  IF(IBUFF(1).EQ.INT(HDH(2,I))) THEN
                     MFLAG=1
                     HDH(1,I)=HDH(1,I)+1000*RBUFF(2)*BHFACT(2,ITHET)
                     GOTO 360
                  ENDIF
 360           CONTINUE
               IF(MFLAG.EQ.0) THEN
                  IF (INT(1000.*RBUFF(2)*BHFACT(2,ITHET)).GT.0) THEN
                     NHHIT = NHHIT + 1
                     HDH(1,NHHIT) = 1000*RBUFF(2)*BHFACT(2,ITHET)
                     HDH(2,NHHIT) = IBUFF(1)
                  ENDIF
               ENDIF
 350        CONTINUE
         ENDIF 
         
      ENDIF
      
      IF (ISIMJD(IDETEC).EQ.ISMRJD) THEN
         
         CALL KZBLOC('MCEESMH',LBANK)
         IF (LBANK.LE.0) THEN
            CALL MSGOUT('GETCAL','MCEESMH does not exist',LBANK)
            IERR = 1
         ELSE 
            CALL KZGET('MCEESMH',0,NBUFF,IBUFF)
            NCALH(7) = IBUFF(1)
            ISTA = NCALH(1)+1
            IEND = NCALH(1)+NCALH(3)
            
            DO 400 IHIT =1, NCALH(7)
               NHIT = NHIT +1
               CALL KZGET('MCEESMH',IHIT,NBUFF,IBUFF)
               IBUFF(1) = IBUFF(1)+ISIGN(1000000,IBUFF(1))
               RCALH(1,NHIT) = RBUFF(2)
               RCALH(2,NHIT) = IBUFF(1)
               RCALH(3,NHIT) = 221.
               ITHET = ABS(MOD(IBUFF(1),1000000)) / 1000
               MFLAG=0
               DO 410 I=ISTA,IEND
                  IF(IBUFF(1).EQ.INT(EMH(2,I))) THEN
                     MFLAG=1
                     EMH(1,I)=EMH(1,I)+1000*RBUFF(2)*EEFACT(1,ITHET)
                     GOTO 410
                  ENDIF
 410           CONTINUE
               IF(MFLAG.EQ.0) THEN
                  IF (INT(1000.*RBUFF(2)*EEFACT(1,ITHET)).GT.0) THEN
                     NEHIT = NEHIT + 1
                     EMH(1,NEHIT) = 1000*RBUFF(2)*EEFACT(1,ITHET)
                     EMH(2,NEHIT) = IBUFF(1)
                  ENDIF
               ENDIF
 400        CONTINUE
         ENDIF
         
         CALL KZBLOC('MCEHSMH',LBANK)
         IF (LBANK.LE.0) THEN
            CALL MSGOUT('GETCAL','MCEHSMH does not exist',LBANK)
            IERR = 1
         ELSE 
            CALL KZGET('MCEHSMH',0,NBUFF,IBUFF)
            NCALH(8) = IBUFF(1)
            ISTA = NCALH(2)+1
            IEND = NCALH(2)+NCALH(4)
            
            DO 450 IHIT =1, NCALH(8)
               NHIT = NHIT +1
               CALL KZGET('MCEHSMH',IHIT,NBUFF,IBUFF)
               IBUFF(1) = IBUFF(1)+ISIGN(1000000,IBUFF(1))
               RCALH(1,NHIT) = RBUFF(2)
               RCALH(2,NHIT) = IBUFF(1)
               RCALH(3,NHIT) = 222.
               ITHET = ABS(MOD(IBUFF(1),1000000)) / 1000
               MFLAG=0
               DO 460 I=ISTA,IEND
                  IF(IBUFF(1).EQ.INT(HDH(2,I))) THEN
                     MFLAG=1
                     HDH(1,I)=HDH(1,I)+1000*RBUFF(2)*EHFACT(2,ITHET)
                     GOTO 460
                  ENDIF
 460           CONTINUE
               IF(MFLAG.EQ.0) THEN
                  IF (INT(1000.*RBUFF(2)*EHFACT(2,ITHET)).GT.0) THEN
                     NHHIT = NHHIT + 1
                     HDH(1,NHHIT) = 1000*RBUFF(2)*EHFACT(2,ITHET)
                     HDH(2,NHHIT) = IBUFF(1)
                  ENDIF
               ENDIF
 450        CONTINUE
         ENDIF
         
      ENDIF
      
      IF (NHIT.GE.MXRCAL) THEN
*** MAYBE data array was broken. ***
         CALL MSGOUT('GETCAL','CAL RAW HITS exceeds the 
     &                MAX',NHIT)
         IERR = 2
         RETURN
      ENDIF
      
      IF (NEHIT.GE.MXECLS) THEN
*** MAYBE data array was broken. ***
         CALL MSGOUT('GETCAL','EMH exceeds the MAX',NEHIT)
         IERR = 3
         RETURN
      ENDIF
      
      IF (NHHIT.GE.MXHCLS) THEN
*** MAYBE data array was broken. ***
         CALL MSGOUT('GETCAL','HDH exceeds the MAX',NHHIT)
         IERR = 4
         RETURN
      ENDIF
      
      NCALH(9) = NHIT
      NEMH = NEHIT
      NHDH = NHHIT
          
      RETURN
      END
